function [P, Q, F, L] = PSMF(x, C, nu, I, P, Q, F, L)
%PROBABILISTIC SPARSE MATRIX FACTORIZATION  --  Q(Y)Q(Z)Q(S)Q(r)
%   Factors X into YZ with non-negativity constraint Y>0, Z>0
%   Y is sparse; row vector y(g,:) has r(g) non-zero entries at s(g,1:r(g))
%   i.e. each row vector x_g is explained by r_g of C possible z_c vectors
%
%
%[P,Q,F,L] = PSMF1(x,C,nu, I, P,Q,F,L)
% x is the 2-D (G-by-T) data set where:
%      G is the number of observed vectors (i.e. #genes)
%      T is the dimension of the observed vectors (i.e. #tissues)
% C is the total number of factors (i.e. #transcription factors)
% nu is the prior on the number of factors explaining each vector
%      nu = [P(r_g)=1  P(r_g=2)    P(r_g=N)]
%      if nu is specified as a scalar, N,  nu = 1./(1:N)
%
% I is the number of EM iterations to compute (default 25)
%
% The return values can be also be passed as initialization parameters
%    to continue aborted computations:
% P contains the following latent variables from the P-distribution:
%      y, z, s, r, and (learned parameter) psi
% Q contains the variational parameters from the Q-distribution:
%      lambda, zeta, phi, sigma, rho
% F is the variational free energy:   D(Q||P) - log P(X)
% L is the complete log likelihood:   log P(X,Y,Z,S,r|Phi)
%
%
% ref: Dueck & Frey: "Probabilistic Sparse Matrix Factorization"
%         University of Toronto technical report PSI-2004-23
%

savecheckpoint = 0; % save a checkpoint every _?_ iterations (0=off)
finalsave = 1; % save workspace upon completion (0=off, 1=on)
nonnegative = 1; % 0=non-negativity thresholding off, 1=on (default)
epsilon = 1e-5; % this is a numerical stability floor for variances (phi, psi)

% Initialize input parameters (if not initialized already)
if nargin < 8 L = -Inf; end; % no error checking on this parameter
if nargin < 7 F = +Inf; end; % no error checking on this parameter
if nargin < 6 Q = struct; end; % no error checking on this parameter
if nargin < 5 P = struct; end; % no error checking on this parameter
if nargin < 4 I = 25; end;
if nargin < 3 error('This function requires at least three input arguments (i.e. x, C, nu)'); end;
if (ndims(x)~=2 || size(x,1)<=1 || size(x,2)<=1) error('Input argument x must be 2-D (G-by-T)'); end;
if (length(nu)==1 && nu>0 && ceil(nu)==floor(nu)) nu=1./(1:nu); end;
if (ndims(nu)~=2 || size(nu,1)~=1 || size(nu,2)<1 || min(nu)<0) error('Input argument nu must be a positive row-vector'); end;
if (length(C) ~= 1 || C(1) < 0 || ceil(C)~=floor(C)) error('Input argument C must be a positive integer'); end;
if (length(nu) > C) error('Input argument nu cannot have higher dimension than C'); end;
if (length(I) ~= 1 || I(1) < 0 || ceil(I)~=floor(I)) error('Input argument I must be a positive integer'); end;

% sort out display of prior P(r)
N = length(nu); nu = nu / sum(nu);
nustr = '[';
if nu(1) == 1;
	nustr = [nustr '1.00]'];
	N = 2; nu = [1 0]; % trick to make sure degenerate case doesn't cause errors
else
	for n = 1:N, nustr = [nustr sprintf('%.2f ',nu(n))]; end; nustr = [nustr(1:end-1) ']'];
end;

% Initialize P- and Q-distributions
                                          [G,T] = size(x);
if (~isfield(P,'y'))      P.y      = zeros(G,C); end;
if (~isfield(P,'z'))      P.z      = zeros(C,T); end;
if (~isfield(P,'s'))      P.s      = zeros(G,N); end;
if (~isfield(P,'r'))      P.r      = zeros(G,1); end;
if (~isfield(P,'psi'))    P.psi    = zeros(G,1); P.psi = std(x,1,2); end;
if (~isfield(Q,'lambda')) Q.lambda = zeros(G,C); Q.lambda = abs(repmat(sqrt(P.psi),[1 C]).*randn(size(Q.lambda))); end;
if (~isfield(Q,'zeta'))   Q.zeta   =  rand(C,T); end;
if (~isfield(Q,'phi'))    Q.phi    =  rand(C,1); end;
if (~isfield(Q,'sigma'))  Q.sigma  = zeros(G,C,N); Q.sigma = rand(size(Q.sigma)); Q.sigma = Q.sigma ./ repmat(sum(Q.sigma,2), [1 C 1]); end;
if (~isfield(Q,'rho'))    Q.rho    = zeros(G,N); Q.rho = repmat(nu, [G 1]); end;

% Display initialization information and progress bar
fprintf('Probabilistic Sparse Matrix Factorization -- Q(Y)Q(Z)Q(S)Q(r)\n');
fprintf('  (%dx%d)=(%dx%d)(%dx%d), nu=%s, %d iterations begun %s @ %s\n', G,T,G,C,C,T, nustr, I, datestr(clock,'ddd'), datestr(clock,'HH:MM:SS'));
fprintf('  {%s}', repmat(' ', [1 I]));

crossterms = calc_crossterms(Q); % initialize the major cached parameter
for i = length(F):I,

	% E-step: update Q.rho
	Q.rho = -permute(cumsum(sum(Q.sigma.*log(1e-50+Q.sigma),2),3),[1 3 2]);
	temp=zeros(G,C);
	for t = 1:T,
		temp = temp + ( -2*Q.lambda.*(x(:,t)*Q.zeta(:,t)') + Q.lambda.^2.*repmat(Q.zeta(:,t)'.^2+Q.phi', [G 1]) );
	end;
	for n = 2:N,
		Q.rho(:,n) = Q.rho(:,n) - 0.5./P.psi.*sum(sum(Q.sigma(:,:,2:n),3).*temp,2);
		for n1=1:n, for n2=(n1+1):n, Q.rho(:,n) = Q.rho(:,n) - 1./P.psi.*crossterms{n1}{n2}; end; end;
	end;
	clear temp;
	Q.rho = repmat(nu./C.^[1:N],[G 1]) .* exp(  Q.rho - repmat(max(Q.rho,[],2),[1 N])  );
	Q.rho = Q.rho ./ repmat(sum(Q.rho,2),[1 N]);
	P.r = argmax(Q.rho, [], 2);

	% E-step: update Q.phi
	Q.phi(:) = 1;
	for n = 1:N,
		Q.phi = Q.phi + sum( Q.lambda.^2./repmat(P.psi,[1 C]).*Q.sigma(:,:,n).*repmat(sum(Q.rho(:,n:N),2),[1 C]) ,1)';
	end;
	Q.phi = 1./Q.phi;
	Q.phi = max(Q.phi, epsilon); % HEURISTIC: variances cannot go lower than epsilon

	% E-step: update Q.zeta
	M = zeros(C,C); V = zeros(C,T);
	for cc = 1:C, for n=1:N, for nn=(n+1):N,
		 M(cc,:) = M(cc,:) + sum(repmat(sum(Q.rho(:,nn:N),2)./P.psi,[1 C]) .* Q.lambda(:,:) .* Q.sigma(:,:,n) .* repmat(Q.lambda(:,cc).*Q.sigma(:,cc,nn),[1 C]), 1);
	end; end; end;
	M = M + M';
	temp = zeros(G,C); for n = 1:N, temp = temp + repmat(sum(Q.rho(:,n:N),2),[1 C]).*Q.sigma(:,:,n); end;
	M = M + diag(sum(Q.lambda.^2./repmat(P.psi,[1 C]).*temp, 1));
	for t = 1:T, V(:,t) = sum(repmat(x(:,t)./P.psi,[1 C]).*Q.lambda.*temp,1)'; end;
	Q.zeta = (M+eye(C))\V;
	clear M V temp;
	if (nonnegative) Q.zeta(find(Q.zeta<0)) = 0; end; % HEURISTIC: negative expression levels not allowed
	P.z = Q.zeta;
	% note that crossterms is stale here, but it doesn't matter since Q.sigma and Q.lambda updates follow

	% E-step: update Q.sigma
	crossterms = zeros(G,C,N); for cc = 1:C,
		crossterms = crossterms + repmat(Q.sigma(:,cc,:),[1 C 1]) .* repmat(Q.lambda.*repmat(Q.lambda(:,cc),[1 C]).*repmat(Q.zeta(cc,:)*Q.zeta',[G 1]), [1 1 N]); end;
	Q.sigma(:) = 0;
	for t = 1:T,
		Q.sigma = Q.sigma - 1/2*repmat( ((repmat(x(:,t),[1 C])-Q.lambda.*repmat(Q.zeta(:,t)',[G 1])).^2 ...
			+ Q.lambda.^2.*repmat(Q.phi',[G 1]))./repmat(P.psi,[1 C]), [1 1 N] );
	end;
	for n = 1:N,
		for nn = 1:N,
			if (nn==n) continue; end;
			Q.sigma(:,:,n) = Q.sigma(:,:,n) - repmat(sum(1e-50+Q.rho(:,max(n,nn):N),2)./(sum(1e-50+Q.rho(:,n:N),2))./P.psi,[1 C]).*crossterms(:,:,nn);
		end;
	end;
	Q.sigma = exp(  Q.sigma - repmat(max(Q.sigma, [], 2), [1 C])  );
	Q.sigma = Q.sigma ./ repmat(sum(Q.sigma, 2), [1 C]);
	crossterms = calc_crossterms(Q);
	P.s = argmax(Q.sigma, [], 2); P.s = permute(P.s, [1 3 2]);

	% M-step: update Q.lambda
	D = zeros(C,1); V = zeros(G,C);
	for t = 1:T,
		D = D + (Q.zeta(:,t).^2 + Q.phi);
		V = V + x(:,t)*Q.zeta(:,t)';
	end;
	temp = zeros(G,C); for n = 1:N, temp = temp + repmat(sum(Q.rho(:,n:N),2),[1 C]).*Q.sigma(:,:,n); end;
	V = V .* temp; D = repmat(D',[G 1]) .* temp;
	D = D + epsilon; % HEURISTIC: weak Gaussian prior on lambda to prevent ill-conditioning
	for g = 1:G,
		M = zeros(C,C);
		for n=1:N, for nn=(n+1):N,
			M = M + sum(Q.rho(g,nn:N))*(Q.sigma(g,:,n)'*Q.sigma(g,:,nn));
		end; end;
		M = (M + M').*(Q.zeta*Q.zeta');
		Q.lambda(g,:) = V(g,:)/(M+diag(D(g,:)));
	end;
	clear M D V temp;
	if (nonnegative) Q.lambda(find(Q.lambda<0)) = 0; end; % HEURISTIC: negative mixing proportions not allowed
	P.y = zeros(G,C); for n = 1:N,
		locs = find(P.r>=n); if ~length(locs) continue; end;
		locs = sub2ind([G C], locs, P.s(locs,n)); P.y(locs) = P.y(locs) + Q.lambda(locs);
	end; clear locs;
	crossterms = calc_crossterms(Q);

	% M-step: update P.psi
	P.psi = -sum(repmat((2:N)-1,[G 1]).*Q.rho(:,2:N),2).*sum(x.^2,2);
    temp = zeros(G,C); for t = 1:T,
        temp = temp + (repmat(x(:,t),[1 C]) - Q.lambda.*repmat(Q.zeta(:,t)', [G 1])).^2 + Q.lambda.^2.*repmat(Q.phi', [G 1]); end;
	for n = 1:N, P.psi = P.psi + sum(Q.rho(:,n:N),2).*sum(Q.sigma(:,:,n).*temp,2); end;
	for n=1:N, for nn=(n+1):N, P.psi = P.psi + 2*sum(Q.rho(:,nn:N),2).*crossterms{n}{nn}; end; end;
	P.psi = P.psi / T;
	clear temp;
	P.psi = max(P.psi, epsilon); % HEURISTIC: variances cannot go lower than epsilon

	if (nargout>=3) F(end+1) = free_energy(x,nu,P,Q); end; % Don't waste time calculating free energy if not requested
	if (nargout>=4) L(end+1) = log_likelihood(x,nu,P); end; % Don't waste time calculating log_likelihood if not requested
% 	if (max(diff(F)) > 0) warning('Non-monotonic free energy detected'); end;
	if (mod(i,savecheckpoint)==0) save(sprintf('PSMF1 checkpoint G=%d,T=%d,C=%d,nu=%s, i=%dof%d.mat', G,T,C,nustr,i,I)); end;
	fprintf('%s{%s%s}', repmat(sprintf('\b'), [1 I+2]), repmat('', [1 i]), repmat(' ', [1 I-i]));
end;
fprintf('\n');

if (nargout>=3 && F(1)==+Inf) F = F(2:end); end; % get rid of initial +Inf padding value
if (nargout>=4 && L(1)==-Inf) L = L(2:end); end; % get rid of initial -Inf padding value
if nu(1) == 1
	N = 1;
	nu = 1;
	Q.rho = Q.rho(:,1);
	P.s = P.s(:,1);
	Q.sigma = Q.sigma(:,:,1);
end;

if (G == 22709)	% built-in GO significance testing on a particular dataset
	load('mouseGO_30-Mar-04');
	load('mouse_profile_23-Dec-03');
	pval = zeros(C,N);
	pind = zeros(C,N);
	for n = 1:N,
		rr = []; cc = [];
		for c = 1:C,
			rr = [rr; intersect(find(P.s(:,n)==c), find(P.r>=n))];
			cc = [cc; repmat(c,[length(rr)-length(cc) 1])];
		end;
		clusters.data = sparse(rr, cc, 1, G, C);
		clusters.rowlabels = Profile.rowlabels;
		clusters.collabels = 1:C;
		Pval = funspec_calculator_v2(clusters, GOdiscrim);
% 		pval(:,n) = -full(max(Pval.data, [], 2));
		[pval(:,n) pind(:,n)] = max(Pval.data, [], 2);
	end;
	pval = -pval;
	clear rr cc clusters Pval;
end;
	
clear t g c cc n nn n1 n2 i crossterms;
if (finalsave) save(sprintf('PSMF1 G=%d,T=%d,C=%d,nu=%s.mat', G,T,C,nustr)); end;
return;
%
%
function crossterms = calc_crossterms(Q)
	[C,T] = size(Q.zeta); [G,N] = size(Q.rho);
	
	outer_zeta = Q.zeta*Q.zeta';
	for n1 = 1:N, for n2 = (n1+1):N,
		crossterms{n1}{n2} = zeros(G,1);
		for c = 1:C,
			crossterms{n1}{n2} = crossterms{n1}{n2} + sum(Q.sigma(:,:,n1).*Q.lambda.*repmat(Q.sigma(:,c,n2).*Q.lambda(:,c),[1 C]).*repmat(outer_zeta(c,:),[G 1]), 2);
		end;
	end; end;
return;
%
%
function F = free_energy(x,nu,P,Q)
	[C,T] = size(Q.zeta); [G N] = size(Q.rho);

	crossterms = calc_crossterms(Q); % you could save time by caching this (make it persistent), but it hinders modularity

 	F = 0;
	for n=1:N, F = F + sum(sum(Q.rho(:,n:N),2).*sum(Q.sigma(:,:,n).*log(1e-50+C*Q.sigma(:,:,n)),2)); end;
	F = F + sum(Q.rho(:).*log(1e-50+Q.rho(:))) - sum(log(1e-50+nu).*sum(Q.rho));
	F = F - T/2*sum(1+log(1e-50+Q.phi(:)));
	F = F + 1/2*sum(Q.zeta(:).^2) + T/2*sum(Q.phi(:));
	F = F + T/2*sum(log(1e-50+2*pi*P.psi));
    temp = zeros(G,C);
    for t = 1:T,
        temp = temp + (repmat(x(:,t),[1 C]) - Q.lambda.*repmat(Q.zeta(:,t)', [G 1])).^2 + Q.lambda.^2.*repmat(Q.phi', [G 1]);
    end;
	for n = 1:N,
		F = F + 1/2*sum(sum(Q.sigma(:,:,n).*temp, 2).*sum(Q.rho(:,n:N),2)./P.psi);
	end;
	F = F - 1/2*sum( sum(repmat(1:(N-1),[G 1]).*Q.rho(:,2:N),2) ./ P.psi .* sum(x.^2,2) );
	for n = 1:N, for nn = (n+1):N,
		F = F + sum(  sum(Q.rho(:,nn:N),2) ./ P.psi .* crossterms{n}{nn}  );
	end; end;
return;
%
%
function L = log_likelihood(x,nu,P)
	[C,T] = size(P.z); [G,N] = size(P.s);

	L = -log(C)*sum(P.r) + sum(log(1e-50+nu(P.r))) - T/2*sum(log(1e-50+2*pi*P.psi)) - 1/2*sum(log(2*pi)+P.z(:).^2);
	L = L - 1/2*sum(sum((x - P.y*P.z).^2,2)./P.psi);
return;
%
%
function arg = argmax(varargin), [junk, arg] = max(varargin{:}); return;
%
%
function arg = argmin(varargin), [junk, arg] = min(varargin{:}); return;
%


[C,T] = size(P.z); [G,N] = size(P.s); % obtain hyperparameters from P-distribution
[C,T] = size(Q.zeta); [G,N] = size(Q.rho); % obtain hyperparameters from Q-distribution
%
% VISUALIZATION ROUTINES
%
set(figure, 'name', 'FREE ENERGY & COMPLETE LOG-LIKELIHOOD', 'NumberTitle', 'off');
	plot(1:length(F), F, 'r.-', 1:length(L), -L, 'b.-');
	title('FREE ENERGY & COMPLETE LOG-LIKELIHOOD'); xlabel('iteration (i)'); legend('D(Q||P)', '-l_C(\theta; X,Z,S,r)');
set(figure, 'name', 'FACTOR COUNT (R) HISTOGRAM', 'NumberTitle', 'off');
	colormap(gray); hist(P.r, [1:N]);
	title('FACTOR COUNT (R) HISTOGRAM'); xlabel('r_g value'); ylabel('count');
set(figure, 'name', 'FACTOR COUNT (rho) ENTROPY HISTOGRAM', 'NumberTitle', 'off'); % requires Q-distribution
	colormap(gray); hist(-sum(Q.rho.*(1e-50+log(Q.rho)),2),25);
	title('SOURCE NUMBER (\rho) ENTROPY HISTOGRAM'); xlabel('H(\rho_g) in nats'); ylabel('count');
set(figure, 'name', 'FACTOR (CLASS) ASSIGNMENT HISTOGRAM', 'NumberTitle', 'off');
	colormap(gray); bar(1:C,hist(P.s,1:C),'stacked'); shading faceted; axis tight;
	title('FACTOR (CLASS) ASSIGNMENT (S) HISTOGRAM'); xlabel('s_{gn} value'); ylabel('count'); legend([repmat('n=',[N 1]) num2str([1:N]')]);
set(figure, 'name', 'FACTOR (CLASS) ENTROPY HISTOGRAM', 'NumberTitle', 'off'); % requires Q-distribution
	colormap(gray); bar(linspace(0,log(C),50),hist(permute(-sum(Q.sigma.*log(1e-50+Q.sigma),2),[1 3 2]),linspace(0,log(C),50)),'stacked'); shading faceted; axis tight;
	title('FACTOR (CLASS) ENTROPY (\sigma) HISTOGRAM'); xlabel('H(\sigma_gn) in nats'); ylabel('count'); legend([repmat('n=',[N 1]) num2str([1:N]')]);
set(figure, 'name', 'DATA SET HISTOGRAM', 'NumberTitle', 'off');
	hist(x(:), 50); title('DATA SET (X) HISTOGRAM'); xlabel('x_{tg} value'); ylabel('count');
set(figure, 'name', 'FACTOR STRENGTH HISTOGRAM', 'NumberTitle', 'off');
	y = P.y(sub2ind([G C], repmat([1:G]',[1 N]),P.s)); for n = 2:N, y(find(P.r<n),n) = NaN; end;
	colormap(gray); bar(0:.2:5,hist(y,0:.2:5),'stacked'); shading faceted; axis tight; clear y;
	title('FACTOR STRENGTH (Y) HISTOGRAM'); xlabel('y_{gs_{gn}} value'); ylabel('count'); legend([repmat('n=',[N 1]) num2str([1:N]')]);
set(figure, 'name', 'FACTORS HISTOGRAM', 'NumberTitle', 'off');
	hist(P.z(:), 50); title('FACTORS (Z) HISTOGRAM'); xlabel('z_{tc} value'); ylabel('count');
set(figure, 'name', 'SENSOR NOISE HISTOGRAM', 'NumberTitle', 'off');
	hist(P.psi(:), 50); title('SENSOR NOISE (\Psi) HISTOGRAM'); xlabel('\psi_g value'); ylabel('count');
%
% FACTORS IMAGE (requires Q-distribution)
%
set(figure, 'name', 'FACTOR (EXPRESSION) VECTORS', 'NumberTitle', 'off');
colormap(gray(256));
image([1:C],[1:T],256*P.z'/2); hold on;
image([1:C],-1,256*Q.phi'/max(Q.phi)); hold on;
A = struct;
A.Box = 'off';
A.Color = 'none';
A.FontSize = 11;
A.GridLineStyle = ':';
A.LineWidth = 3;
A.XColor = 'k';
A.XGrid = 'on';
A.XAxisLocation = 'bottom';
A.XMinorGrid = 'off';
A.XMinorTick = 'off';
A.XTick = [];
A.XTickLabel = [];
A.YColor = 'k';
A.YGrid= 'off';
A.YAxisLocation = 'left';
A.YMinorGrid = 'off';
A.YMinorTick = 'off';
A.YTick = [-1 T/2];
A.YTickLabel = {'Uncertainty (phi)', 'Vectors (z_c)'};
set(gca,A); axis equal tight;
title('FACTOR (EXPRESSION) VECTORS', 'FontSize', 16, 'FontWeight', 'bold');
xlabel('\leftarrow FACTORS \rightarrow', 'FontSize', 10, 'FontWeight', 'normal', 'Color', 'k');
% ylabel('\leftarrow TISSUES \rightarrow', 'FontSize', 10, 'FontWeight', 'normal', 'Color', 'k');
%
% RESULTS EXPLORER INTERFACE
%
border = 7; randmap = rand([1 C+1 3]); randmap(1,1,:) = 1;
set(figure, 'Name', 'PSMF RESULTS EXPLORER', 'NumberTitle', 'off', 'Renderer', 'ZBuffer');
colormap(gray(256));
A = struct; A.YTick = []; cur_y = 1;
x_hat = P.y*P.z;
x_res = x-x_hat;
junk=[]; for n=1:N, junk=[junk (P.r>=n).*P.s(:,n)]; end; junk = [junk P.psi]; [junk,p] = sortrows(junk); clear junk;
set(image(1:G,cur_y,256*P.psi(p)'/.5), 'Tag', 'Noise (Psi)'); hold on;
	A.YTick = [A.YTick cur_y]; cur_y = cur_y + 2;
set(image(1:G,cur_y+[0:T-1],256*x(p,:)'/3), 'Tag', 'Data (X)');
	A.YTick = [A.YTick cur_y+T/2]; cur_y = cur_y + T-1 + border;
set(image(1:G,cur_y, 256*P.r(p)'/N), 'Tag', '#sources (r)');
	A.YTick = [A.YTick cur_y]; cur_y = cur_y + 3;
set(image(1:G,cur_y, 256*sum(-Q.rho(p,n).*log(1e-50+Q.rho(p,n)),2)'/log(N)), 'Tag', sprintf('rho entropy',n));
	A.YTick = [A.YTick cur_y]; cur_y = cur_y + border;
for n = 1:N,
	set(image(1:G,cur_y,randmap(1,1+(P.r(p)>=n).*P.s(p,n),:)), 'Tag', sprintf('%d factor (s)',n));
		A.YTick = [A.YTick cur_y]; cur_y = cur_y + 3;
	set(image(1:G,cur_y,256*(P.r(p)'<n)+256*P.y(sub2ind([G C],p,P.s(p,n)))'/2), 'Tag', sprintf('%d strength (y)',n)); % TODO: fix P.lambda here
		A.YTick = [A.YTick cur_y]; cur_y = cur_y + 3;
	set(image(1:G,cur_y,256*(P.r(p)<n)' + 256*sum(-Q.sigma(p,:,n).*log(1e-50+Q.sigma(p,:,n)),2)'/log(C)), 'Tag', sprintf('%d entropy',n));
		A.YTick = [A.YTick cur_y]; cur_y = cur_y + border;
end;
set(image(1:G,cur_y+[0:T-1],256*x_hat(p,:)'/3), 'Tag', 'Predicted (X^)');
	A.YTick = [A.YTick cur_y+T/2]; cur_y = cur_y + T-1 + border;
set(image(1:G,cur_y+[0:T-1],256*abs(x_res(p,:))'/3), 'Tag', 'Residual (|X-X^|)');
	A.YTick = [A.YTick cur_y+T/2]; cur_y = cur_y + T-1 + border;
A.Tag = 'RESULTS image';
A.Box = 'off';
A.Color = 'none';
A.FontSize = 10;
A.FontWeight = 'bold';
A.GridLineStyle = '--';
A.LineWidth = 2;
A.XColor = [.5 .5 1]; % KEY PARAMETER
A.XGrid = 'on';
A.XAxisLocation = 'top';
A.XMinorGrid = 'off';
A.XMinorTick = 'off';
A.XTick = .5+find(diff(P.s(p,1))); %.5+cumsum(hist(P.s(:,1),C)); A.XTick = A.XTick(1:end-1);
A.XTickLabel = 2:C;
A.XMinorGrid = 'off';
A.XMinorTick = 'off';
A.YColor = 'k';
A.YGrid= 'off';
A.YAxisLocation = 'left';
A.YMinorGrid = 'off';
A.YMinorTick = 'off';
A.YTickLabel = get(flipud(get(gca,'Children')),'Tag');
set(gca,A); axis tight;
title('PSMF RESULTS EXPLORER', 'FontSize', 16, 'FontWeight', 'bold');
clear border randmap cur_y n p A;
zoom2cursor; % special modifications to file -- available at www.psi.toronto.edu/~delbert/zoom2cursor.m