function [P, Q, F, L] = PSMF(x, C, nu, I, P, Q, F, L)
%PROBABILISTIC SPARSE MATRIX FACTORIZATION  --  Q(Z)Q(Y,S,r)
%   Factors X into YZ with non-negativity constraint Y>0, Z>0
%   Y is sparse; row vector y(g,:) has r(g) non-zero entries at s(g,1:r(g))
%   i.e. each row vector x_g is explained by r_g of C possible z_c vectors
%
%
%[P,Q,F,L] = PSMF3(x,C,nu, I, P,Q,F,L)
% x is the 2-D (G-by-T) data set where:
%      G is the number of observed vectors (i.e. #genes)
%      T is the dimension of the observed vectors (i.e. #tissues)
% C is the total number of factors (i.e. #transcription factors)
% nu is the prior on the number of factors explaining each vector
%      nu = [P(r_g)=1  P(r_g=2)    P(r_g=N)]
%      if nu is specified as a scalar, N,  nu = 1./(1:N)
%
% I is the number of EM iterations to compute (default 25)
%
% The return values can be also be passed as initialization parameters
%    to continue aborted computations:
% P contains the following latent variables from the P-distribution:
%      y, z, s, r, and (learned parameter) psi
% Q contains the variational parameters from the Q-distribution:
%      lambda, zeta, phi, sigma
% F is the variational free energy:   D(Q||P) - log P(X)
% L is the complete log likelihood:   log P(X,Y,Z,S,r|Phi)
%
%
% ref: Dueck & Frey: "Probabilistic Sparse Matrix Factorization"
%         University of Toronto technical report PSI-2004-23
%

savecheckpoint = 0; % save a checkpoint every _?_ iterations (0=off)
finalsave = 1; % save workspace upon completion (0=off, 1=on)
nonnegative = 1; % 0=non-negativity thresholding off, 1=on (default)
epsilon = 1e-5; % this is a numerical stability floor for variances (phi, psi)

% Initialize input parameters (if not initialized already)
if nargin < 8 L = -Inf; end; % no error checking on this parameter
if nargin < 7 F = +Inf; end; % no error checking on this parameter
if nargin < 6 Q = struct; end; % no error checking on this parameter
if nargin < 5 P = struct; end; % no error checking on this parameter
if nargin < 4 I = 25; end;
if nargin < 3 error('This function requires at least three input arguments (i.e. x, C, nu)'); end;
if (ndims(x)~=2 || size(x,1)<=1 || size(x,2)<=1) error('Input argument x must be 2-D (G-by-T)'); end;
if (length(nu)==1 && nu>0 && ceil(nu)==floor(nu)) nu=1./(1:nu); end;
if (ndims(nu)~=2 || size(nu,1)~=1 || size(nu,2)<1 || min(nu)<0) error('Input argument nu must be a positive row-vector'); end;
if (length(C) ~= 1 || C(1) < 0 || ceil(C)~=floor(C)) error('Input argument C must be a positive integer'); end;
if (length(nu) > C) error('Input argument nu cannot have higher dimension than C'); end;
if (length(I) ~= 1 || I(1) < 0 || ceil(I)~=floor(I)) error('Input argument I must be a positive integer'); end;

% sort out display of prior P(r)
N = length(nu); nu = nu / sum(nu);
nustr = '[';
if nu(1) == 1;
	nustr = [nustr '1.00]'];
	N = 2; nu = [1 0]; % trick to make sure degenerate case doesn't cause errors
else
	for n = 1:N, nustr = [nustr sprintf('%.2f ',nu(n))]; end; nustr = [nustr(1:end-1) ']'];
end;
CN=0; for n=1:N, CN = CN + nchoosek(C,n); end;

% Initialize P- and Q-distributions
                                          [G,T] = size(x);
if (~isfield(P,'y'))      P.y      = zeros(G,C); end;
if (~isfield(P,'z'))      P.z      = zeros(C,T); end;
if (~isfield(P,'s'))      P.s      = zeros(G,N); end;
if (~isfield(P,'r'))      P.r      = zeros(G,1); end;
if (~isfield(P,'psi'))    P.psi    = zeros(G,1); P.psi = std(x,1,2); end;
if (~isfield(Q,'lambda')) Q.lambda = zeros(G,N,CN); Q.lambda = abs(repmat(sqrt(P.psi),[1 N CN]).*randn(size(Q.lambda))); end;
if (~isfield(Q,'zeta'))   Q.zeta   =  rand(C,T); end;
if (~isfield(Q,'phi'))    Q.phi    =  rand(C,1); end;
if (~isfield(Q,'sigma'))  Q.sigma  = zeros(G,CN); Q.sigma = rand(size(Q.sigma)); Q.sigma = Q.sigma ./ repmat(sum(Q.sigma,2), [1 CN]); end;

% initialize factorial translation indices
CNc = zeros(CN,N);
CNc(1:C,1) = (1:C)';
cn=C+1; for n = 2:N,
	powvector = C.^((n-1):-1:0);
	for c = 1:(C^n),
		inds = mod(floor(c./powvector),C);
		if (max(diff(inds)) < 0)
			CNc(cn,1:n) = 1+inds;
			cn = cn + 1;
		end;
	end;
end;
CNc = sortrows(CNc,N:-1:1); clear powvector inds;
CNc(find(CNc==0)) = NaN; CNc = sort(CNc,2); CNc(find(isnan(CNc))) = 0;
CNn = sum(CNc~=0,2);
Q.CNc = CNc; Q.CNn = CNn;

% Display initialization information and progress bar
fprintf('Probabilistic Sparse Matrix Factorization -- Q(Z)Q(Y,S,r)\n');
fprintf('  (%dx%d)=(%dx%d)(%dx%d), nu=%s, %d iterations begun %s @ %s\n', G,T,G,C,C,T, nustr, I, datestr(clock,'ddd'), datestr(clock,'HH:MM:SS'));
fprintf('  {%s}', repmat(' ', [1 I]));

for i = length(F):I,

	% E-step: update Q.phi
	Q.phi(:) = 1;
	for cn = 1:CN, n = CNn(cn); c = CNc(cn,1:n);
		Q.phi(c) = Q.phi(c) + sum(Q.lambda(:,1:n,cn).^2.*repmat(Q.sigma(:,cn)./P.psi,[1 n]),1)';
	end;
	Q.phi = 1./Q.phi;
	Q.phi = max(Q.phi, epsilon); % HEURISTIC: variances cannot go lower than epsilon

	% E-step: update Q.zeta
	M = eye(C,C); V = zeros(C,T);
	for cn = 1:CN,
		for n = 1:CNn(cn), c = CNc(cn,n);
			V(c,:) = V(c,:) + (x'*(Q.sigma(:,cn).*Q.lambda(:,n,cn)./P.psi))';
			M(c,c) = M(c,c) + sum(Q.sigma(:,cn)./P.psi.*Q.lambda(:,n,cn).^2, 1);
			for nn = (n+1):CNn(cn), cc = CNc(cn,nn);
				M(c,cc) = M(c,cc) + sum(Q.sigma(:,cn)./P.psi.*Q.lambda(:,n,cn).*Q.lambda(:,nn,cn), 1);
			end;
		end;
	end;
	M = M + triu(M,1)';
	Q.zeta = M\V;
	if (nonnegative) Q.zeta(find(Q.zeta<0)) = 0; end; % HEURISTIC: negative expression levels not allowed
	P.z = Q.zeta;

	% E-step: update Q.sigma
	Q.sigma(:) = 0;
	for cn = 1:CN, n = CNn(cn); c = CNc(cn,1:n);
		Q.sigma(:,cn) = sum((x - Q.lambda(:,1:n,cn)*Q.zeta(c,:)).^2, 2) + T*(Q.lambda(:,1:n,cn).^2)*Q.phi(c,:);
		Q.sigma(:,cn) = -.5 * Q.sigma(:,cn) ./ P.psi;
	end;
	Q.sigma = exp(  Q.sigma - repmat(max(Q.sigma, [], 2), [1 CN])  );
	for n = 1:N, Q.sigma(:,find(CNn==n)) = Q.sigma(:,find(CNn==n)) * nu(n)/nchoosek(C,n); end;
	Q.sigma = Q.sigma ./ repmat(sum(Q.sigma, 2), [1 CN]);
	P.s = CNc(argmax(Q.sigma, [], 2), :);
	P.r = CNn(argmax(Q.sigma, [], 2));

	% M-step: update Q.lambda
	for g = 1:G,
		for cn = 1:CN,
			n = CNn(cn); c = CNc(cn,1:n);
			V = x(g,:) * Q.zeta(c,:)';
			M = Q.zeta(c,:)*Q.zeta(c,:)' + T*diag(Q.phi(c));
			Q.lambda(g,1:n,cn) = V/M;
		end;
	end;
	if (nonnegative) Q.lambda(find(Q.lambda<0)) = 0; end; % HEURISTIC: negative mixing proportions not allowed
	P.y = zeros(G,C);
	for g = 1:G,
		cn = argmax(Q.sigma(g,:)); n = CNn(cn); c = CNc(cn,1:n);
		P.y(g,c) = Q.lambda(g,1:n,cn);
	end;

	% M-step: update P.psi
	P.psi(:) = 0;
	for cn = 1:CN, n = CNn(cn); c = CNc(cn,1:n);
		P.psi = P.psi + Q.sigma(:,cn).*(sum((x - Q.lambda(:,1:n,cn)*Q.zeta(c,:)).^2, 2) + T*(Q.lambda(:,1:n,cn).^2)*Q.phi(CNc(cn,1:n),:));
	end;
	P.psi = P.psi / T;
	P.psi = max(P.psi, epsilon); % HEURISTIC: variances cannot go lower than epsilon

	if (nargout>=3) F(end+1) = free_energy(x,nu,P,Q); end; % Don't waste time calculating free energy if not requested
	if (nargout>=4) L(end+1) = log_likelihood(x,nu,P); end; % Don't waste time calculating log_likelihood if not requested
% 	if (max(diff(F)) > 0) warning('Non-monotonic free energy detected'); end;
	if (mod(i,savecheckpoint)==0) save(sprintf('PSMF3 checkpoint G=%d,T=%d,C=%d,nu=%s, i=%dof%d.mat', G,T,C,nustr,i,I)); end;
	fprintf('%s{%s%s}', repmat(sprintf('\b'), [1 I+2]), repmat('', [1 i]), repmat(' ', [1 I-i]));
end;
fprintf('\n');

if (nargout>=3 && F(1)==+Inf) F = F(2:end); end; % get rid of initial +Inf padding value
if (nargout>=4 && L(1)==-Inf) L = L(2:end); end; % get rid of initial -Inf padding value
if nu(1) == 1
	CN = C;
	N = 1;
	nu = 1;
	P.s = P.s(:,1);
	Q.sigma = Q.sigma(:,1:C);
	Q.lambda = Q.lambda(:,1,1:C);
end;

clear t g c cc n nn i; Q = rmfield(Q, {'CNc','CNn'});
if (finalsave) save(sprintf('PSMF3 G=%d,T=%d,C=%d,nu=%s.mat', G,T,C,nustr)); end;
return;
%
%
function F = free_energy(x,nu,P,Q)
	[C,T] = size(Q.zeta); [G CN] = size(Q.sigma);
	CNc = Q.CNc; CNn = Q.CNn;
	
	F = 0;
	F = F - T/2*sum(1+log(1e-50+Q.phi(:)));
	F = F + 1/2*sum(Q.zeta(:).^2) + T/2*sum(Q.phi(:));
	F = F + T/2*sum(log(1e-50+2*pi*P.psi(:)));

	for cn = 1:CN, n = CNn(cn); c = CNc(cn,1:n);
		F = F + sum(Q.sigma(:,cn).*log(1e-50+Q.sigma(:,cn)/(nu(n)/nchoosek(C,n))));
		F = F + 1/2*sum(Q.sigma(:,cn)./P.psi.*(sum((x-Q.lambda(:,1:n,cn)*Q.zeta(c,:)).^2,2) + T*(Q.lambda(:,1:n,cn).^2)*Q.phi(c)));
	end;
return;
%
%
function L = log_likelihood(x,nu,P)
	[C,T] = size(P.z); [G,N] = size(P.s);

	L = -log(C)*sum(P.r) + sum(log(1e-50+nu(P.r))) - T/2*sum(log(1e-50+2*pi*P.psi)) - 1/2*sum(log(2*pi)+P.z(:).^2);
	L = L - 1/2*sum(sum((x - P.y*P.z).^2,2)./P.psi);
return;
%
%
function arg = argmax(varargin), [junk, arg] = max(varargin{:}); return;
% 
% 
function arg = argmin(varargin), [junk, arg] = min(varargin{:}); return;


[C,T] = size(P.z); [G,N] = size(P.s); % obtain hyperparameters from P-distribution
[C,T] = size(Q.zeta); [G CN] = size(Q.sigma); % obtain hyperparameters from Q-distribution
%
% VISUALIZATION ROUTINES
%
set(figure, 'name', 'FREE ENERGY & COMPLETE LOG-LIKELIHOOD', 'NumberTitle', 'off');
	plot(1:length(F), F, 'r.-', 1:length(L), -L, 'b.-');
	title('FREE ENERGY & COMPLETE LOG-LIKELIHOOD'); xlabel('iteration (i)'); legend('D(Q||P)', '-l_C(\theta; X,Z,S,r)');
set(figure, 'name', 'FACTOR COUNT (R) HISTOGRAM', 'NumberTitle', 'off'); % add nu numbers here
	colormap(gray); hist(P.r, [1:N]);
	title('FACTOR COUNT (R) HISTOGRAM'); xlabel('r_g value'); ylabel('count');
set(figure, 'name', 'FACTOR (CLASS) ASSIGNMENT HISTOGRAM', 'NumberTitle', 'off');
	s = P.s; s(find(~s)) = NaN; hist(s(:),1:C); clear s; axis tight; 
	title('FACTOR (CLASS) ASSIGNMENT (S) HISTOGRAM'); xlabel('s_{gn} value'); ylabel('count');
set(figure, 'name', 'FACTOR (CLASS) ENTROPY HISTOGRAM', 'NumberTitle', 'off');
	colormap(gray); hist(-sum(Q.sigma.*log(1e-50+Q.sigma),2),linspace(0,log(CN),50)); axis tight;
	title('FACTOR (CLASS) ENTROPY (\sigma) HISTOGRAM'); xlabel('H(\sigma_gn) in nats'); ylabel('count');
set(figure, 'name', 'DATA SET HISTOGRAM', 'NumberTitle', 'off');
	hist(x(:), 50); title('DATA SET (X) HISTOGRAM'); xlabel('x_{tg} value'); ylabel('count');
set(figure, 'name', 'FACTOR STRENGTH HISTOGRAM', 'NumberTitle', 'off');
	y = P.y; y(find(~y)) = NaN; colormap(gray); hist(P.y(:),0:.2:5); axis tight; clear y;
	title('FACTOR STRENGTH (Y) HISTOGRAM'); xlabel('y_{gs_{gn}} value'); ylabel('count'); legend([repmat('n=',[N 1]) num2str([1:N]')]);
set(figure, 'name', 'FACTORS HISTOGRAM', 'NumberTitle', 'off');
	hist(P.z(:), 50); title('FACTORS (Z) HISTOGRAM'); xlabel('z_{tc} value'); ylabel('count');
set(figure, 'name', 'SENSOR NOISE HISTOGRAM', 'NumberTitle', 'off');
	hist(P.psi(:), 50); title('SENSOR NOISE (\Psi) HISTOGRAM'); xlabel('\psi_g value'); ylabel('count');
%
% FACTORS IMAGE
%
set(figure, 'name', 'FACTOR (EXPRESSION) VECTORS', 'NumberTitle', 'off');
colormap(gray(256));
image([1:C],[1:T],256*P.z'/2); hold on;
image([1:C],-1,256*Q.phi'/max(Q.phi)); hold on;
A = struct;
A.Box = 'off';
A.Color = 'none';
A.FontSize = 11;
A.GridLineStyle = ':';
A.LineWidth = 3;
A.XColor = 'k';
A.XGrid = 'on';
A.XAxisLocation = 'bottom';
A.XMinorGrid = 'off';
A.XMinorTick = 'off';
A.XTick = [];
A.XTickLabel = [];
A.YColor = 'k';
A.YGrid= 'off';
A.YAxisLocation = 'left';
A.YMinorGrid = 'off';
A.YMinorTick = 'off';
A.YTick = [-1 T/2];
A.YTickLabel = {'Uncertainty (phi)', 'Vectors (z_c)'};
set(gca,A); axis equal tight;
title('FACTOR (EXPRESSION) VECTORS', 'FontSize', 16, 'FontWeight', 'bold');
xlabel('\leftarrow FACTORS \rightarrow', 'FontSize', 10, 'FontWeight', 'normal', 'Color', 'k');
% ylabel('\leftarrow TISSUES \rightarrow', 'FontSize', 10, 'FontWeight', 'normal', 'Color', 'k');
%
% RESULTS EXPLORER INTERFACE
%
border = 7; randmap = rand([1 C+1 3]); randmap(1,1,:) = 1;
set(figure, 'Name', 'PSMF RESULTS EXPLORER', 'NumberTitle', 'off', 'Renderer', 'ZBuffer');
colormap(gray(256));
A = struct; A.YTick = []; cur_y = 1;
x_hat = P.y*P.z;
x_res = x-x_hat;
junk=[]; for n=1:N, junk=[junk (P.r>=n).*P.s(:,n)]; end; junk = [junk P.psi]; [junk,p] = sortrows(junk); clear junk;
set(image(1:G,cur_y,256*P.psi(p)'/.5), 'Tag', 'Noise (Psi)'); hold on;
	A.YTick = [A.YTick cur_y]; cur_y = cur_y + 2;
set(image(1:G,cur_y+[0:T-1],256*x(p,:)'/3), 'Tag', 'Data (X)');
	A.YTick = [A.YTick cur_y+T/2]; cur_y = cur_y + T-1 + border;
set(image(1:G,cur_y, 256*P.r(p)'/N), 'Tag', '#sources (r)');
	A.YTick = [A.YTick cur_y]; cur_y = cur_y + 3;
set(image(1:G,cur_y, 256*sum(-Q.sigma.*log(1e-50+Q.sigma),2)'/log(N)), 'Tag', sprintf('{sigma,rho} entropy',n));
	A.YTick = [A.YTick cur_y]; cur_y = cur_y + border;
for n = 1:N,
	set(image(1:G,cur_y,randmap(1,1+(P.r(p)>=n).*P.s(p,n),:)), 'Tag', sprintf('%d factor (s)',n));
		A.YTick = [A.YTick cur_y]; cur_y = cur_y + 3;
	s=P.s; s(find(s==0))=1; set(image(1:G,cur_y,256*(P.r(p)'<n)+256*P.y(sub2ind([G C],p,s(p,n)))'/2), 'Tag', sprintf('%d strength (y)',n)); clear s;
		A.YTick = [A.YTick cur_y]; cur_y = cur_y + 3;
end;
set(image(1:G,cur_y+[0:T-1],256*x_hat(p,:)'/3), 'Tag', 'Predicted (X^)');
	A.YTick = [A.YTick cur_y+T/2]; cur_y = cur_y + T-1 + border;
set(image(1:G,cur_y+[0:T-1],256*abs(x_res(p,:))'/3), 'Tag', 'Residual (|X-X^|)');
	A.YTick = [A.YTick cur_y+T/2]; cur_y = cur_y + T-1 + border;
A.Tag = 'RESULTS image';
A.Box = 'off';
A.Color = 'none';
A.FontSize = 10;
A.FontWeight = 'bold';
A.GridLineStyle = '--';
A.LineWidth = 2;
A.XColor = [.5 .5 1]; % KEY PARAMETER
A.XGrid = 'on';
A.XAxisLocation = 'top';
A.XMinorGrid = 'off';
A.XMinorTick = 'off';
A.XTick = .5+find(diff(P.s(p,1))); %.5+cumsum(hist(P.s(:,1),C)); A.XTick = A.XTick(1:end-1);
A.XTickLabel = 2:C;
A.XMinorGrid = 'off';
A.XMinorTick = 'off';
A.YColor = 'k';
A.YGrid= 'off';
A.YAxisLocation = 'left';
A.YMinorGrid = 'off';
A.YMinorTick = 'off';
A.YTickLabel = get(flipud(get(gca,'Children')),'Tag');
set(gca,A); axis tight;
title('PSMF RESULTS EXPLORER', 'FontSize', 16, 'FontWeight', 'bold');
clear border randmap cur_y n p A;
zoom2cursor; % special modifications to file -- available at www.psi.toronto.edu/~delbert/zoom2cursor.m