function [P, L] = SMF(x, C, nu, I, P, L)
%SPARSE MATRIX FACTORIZATION (iterated conditional modes)
%   Factors X into YZ with non-negativity constraint Y>0, Z>0
%   Y is sparse; row vector y(g,:) has r(g) non-zero entries at s(g,1:r(g))
%   i.e. each row vector x_g is explained by r_g of C possible z_c vectors
%
%
%[P,L] = SMF(x,C,nu, I, P,L)
% x is the 2-D (G-by-T) data set where:
%      G is the number of observed vectors (i.e. #genes)
%      T is the dimension of the observed vectors (i.e. #tissues)
% C is the total number of factors (i.e. #transcription factors)
% nu is the prior on the number of factors explaining each vector
%      nu = [P(r_g)=1  P(r_g=2)    P(r_g=N)]
%      if nu is specified as a scalar, N,  nu = 1./(1:N)
%
% I is the number of MM iterations to compute (default 25)
%
% The return values can be also be passed as initialization parameters
%    to continue aborted computations:
% P contains the following latent variables from the P-distribution:
%      y, z, s, r, and (learned parameter) psi
% L is the complete log likelihood:   log P(X,Y,Z,S,r|Phi)
%
%
% ref: Dueck & Frey: "Probabilistic Sparse Matrix Factorization"
%         University of Toronto technical report PSI-2004-23
%

savecheckpoint = 0; % save a checkpoint every _?_ iterations (0=off)
finalsave = 1; % save workspace upon completion (0=off, 1=on)
nonnegative = 1; % 0=non-negativity thresholding off, 1=on (default)
epsilon = 1e-5; % this is a numerical stability floor for variances (phi, psi)

% Initialize input parameters (if not initialized already)
if nargin < 6 L = -Inf; end; % no error checking on this parameter
if nargin < 5 P = struct; end; % no error checking on this parameter
if nargin < 4 I = 25; end;
if nargin < 3 error('This function requires at least three input arguments (i.e. x, C, nu)'); end;
if (ndims(x)~=2 || size(x,1)<=1 || size(x,2)<=1) error('Input argument x must be 2-D (G-by-T)'); end;
if (length(nu)==1 && nu>0 && ceil(nu)==floor(nu)) nu=1./(1:nu); end;
if (ndims(nu)~=2 || size(nu,1)~=1 || size(nu,2)<1 || min(nu)<0) error('Input argument nu must be a positive row-vector'); end;
if (length(C) ~= 1 || C(1) < 0 || ceil(C)~=floor(C)) error('Input argument C must be a positive integer'); end;
if (length(nu) > C) error('Input argument nu cannot have higher dimension than C'); end;
if (length(I) ~= 1 || I(1) < 0 || ceil(I)~=floor(I)) error('Input argument I must be a positive integer'); end;

% sort out display of prior P(r)
N = length(nu); nu = nu / sum(nu);
nustr = '['; for n = 1:N, nustr = [nustr sprintf('%.2f ',nu(n))]; end; nustr = [nustr(1:end-1) ']'];

% Initialize P- and Q-distributions
                                          [G,T] = size(x);
if (~isfield(P,'y'))      P.y      =  rand(G,C); end;
if (~isfield(P,'z'))      P.z      =  rand(C,T); end;
if (~isfield(P,'s'))      P.s      =  rand(G,N); P.s = ceil(C*P.s); end;
if (~isfield(P,'r'))      P.r      =  rand(G,1); P.r = ceil(N*P.r); end;
if (~isfield(P,'psi'))    P.psi    = zeros(G,1); P.psi = std(x,1,2); end;

% Display initialization information and progress bar
fprintf('Sparse Matrix Factorization (iterated conditional modes)\n');
fprintf('  (%dx%d)=(%dx%d)(%dx%d), nu=%s, %d iterations begun %s @ %s\n', G,T,G,C,C,T, nustr, I, datestr(clock,'ddd'), datestr(clock,'HH:MM:SS'));
fprintf('  {%s}', repmat(' ', [1 I]));

for i = length(L):I,
	% update P.s
	for n = 1:max(P.r),
		s = zeros(G,C);
		for nn=1:max(P.r),
			if (nn==n) continue; end;
			s(sub2ind([G C], find(P.r>=nn), P.s(find(P.r>=nn),nn))) = 1;
		end;
		x_hat = (P.y.*s)*P.z;
		x_diff = zeros(G,C);
		for t = 1:T,
			x_diff = x_diff + (repmat(x(:,t), [1 C]) - repmat(x_hat(:,t), [1 C])  -  P.y .* repmat(P.z(:,t)', [G 1])).^2;
		end;
		P.s(:,n) = argmin(x_diff,[],2);
	end;
	clear x_hat x_diff s;

	% update P.r
	r = zeros(G, N);
	x_diff = x;
	for n = 1:N,
		x_diff = x_diff - repmat(P.y(sub2ind([G C], (1:G)', P.s(1:G,n))),[1 T]) .* P.z(P.s(1:G,n), 1:T);
		r(:,n) = 1/2*sum(x_diff.^2,2)./P.psi - log(nu(n)/C^n);
	end;
	P.r = argmin(r,[],2);
	clear r x_diff;

	% update P.z
	s = zeros(G,C); for n=1:max(P.r), s(sub2ind([G C], find(P.r>=n), P.s(find(P.r>=n),n))) = 1; end;
	V = zeros(C,T); for t = 1:T, V(:,t) = ((x(:,t)./P.psi)' * (s.*P.y))'; end;
	M = eye(C) + ((s.*P.y)./repmat(P.psi,[1 C]))'*(s.*P.y);
	P.z = M\V;
	if (nonnegative) P.z(find(P.z<0)) = 0; end; % HEURISTIC: negative expression levels not allowed

	% M-step: update Q.lambda (a.k.a. P.y)
	for g = 1:G,
		s = P.s(g,1:P.r(g));
		M = P.z(s,:)*P.z(s,:)' + epsilon*eye(length(s));
		V = x(g,:)*P.z(s,:)';
		P.y(g,s) = V/M;
	end;
	if (nonnegative) P.y(find(P.y<0)) = 0; end; % HEURISTIC: negative mixing proportions not allowed
	clear M V;

	% M-step: update P.psi
	s = zeros(G,C); for n=1:max(P.r), s(sub2ind([G C], find(P.r>=n), P.s(find(P.r>=n),n))) = 1; end;
	P.psi = mean((x - (P.y.*s)*P.z).^2,2);
	P.psi = max(P.psi, epsilon); % HEURISTIC: variances cannot go lower than epsilon

	if (nargout>=2) L(end+1) = log_likelihood(x,nu,P); end; % Don't waste time calculating log_likelihood if not requested
% 	if (max(diff(L)) > 0) warning('Non-monotonic complete log likelihood detected'); end;
	if (mod(i,savecheckpoint)==0) save(sprintf('SMF checkpoint G=%d,T=%d,C=%d,nu=%s, i=%dof%d.mat', G,T,C,nustr,i,I)); end;
	fprintf('%s{%s%s}', repmat(sprintf('\b'), [1 I+2]), repmat('', [1 i]), repmat(' ', [1 I-i]));
end;
fprintf('\n');
	
if (nargout>=2 && L(1)==-Inf) L = L(2:end); end; % get rid of initial -Inf padding value

if (G == 22709)	% built-in GO significance testing on a particular dataset
	load('mouseGO_30-Mar-04');
	load('mouse_profile_23-Dec-03');
	pval = zeros(C,N);
	for n = 1:N,
		rr = []; cc = [];
		for c = 1:C,
			rr = [rr; intersect(find(P.s(:,n)==c), find(P.r>=n))];
			cc = [cc; repmat(c,[length(rr)-length(cc) 1])];
		end;
		clusters.data = sparse(rr, cc, 1, G, C);
		clusters.rowlabels = Profile.rowlabels;
		clusters.collabels = 1:C;
		Pval = funspec_calculator_v2(clusters, GOdiscrim);
% 		pval(:,n) = -full(max(Pval.data, [], 2));
		[pval(:,n) pind(:,n)] = max(Pval.data, [], 2);
	end;
	pval = -pval;
	clear rr cc clusters Pval;
end;

P.y = s.*P.y; % Finally convert point estimate Q.lambda into sparse P.y
clear t g c c1 n nn n1 n2 i;
if (finalsave) save(sprintf('SMF G=%d,T=%d,C=%d,nu=%s.mat', G,T,C,nustr)); end;
return;
%
%
function L = log_likelihood(x,nu,P)
	[C,T] = size(P.z); [G,N] = size(P.s);
	
	s = zeros(G,C); for n=1:max(P.r), s(sub2ind([G C], find(P.r>=n), P.s(find(P.r>=n),n))) = 1; end;
	L = -log(C)*sum(P.r) + sum(log(1e-50+nu(P.r))) - T/2*sum(log(1e-50+2*pi*P.psi)) - 1/2*sum(log(2*pi)+P.z(:).^2);
	L = L - 1/2*sum(sum((x - (s.*P.y)*P.z).^2,2)./P.psi);
return;
%
%
function arg = argmax(varargin), [junk, arg] = max(varargin{:}); return;
% 
% 
function arg = argmin(varargin), [junk, arg] = min(varargin{:}); return;


[C,T] = size(P.z); [G,N] = size(P.s); % obtain hyperparameters from P-distribution
%
% VISUALIZATION ROUTINES
%
set(figure, 'name', 'COMPLETE LOG-LIKELIHOOD', 'NumberTitle', 'off');
	plot(L, 'b.-');
	title('COMPLETE LOG-LIKELIHOOD'); xlabel('iteration (i)');
set(figure, 'name', 'FACTOR COUNT (R) HISTOGRAM', 'NumberTitle', 'off');
	colormap(gray); hist(P.r, [1:N]);
	title('FACTOR COUNT (R) HISTOGRAM'); xlabel('r_g value'); ylabel('count');
set(figure, 'name', 'FACTOR (CLASS) ASSIGNMENT HISTOGRAM', 'NumberTitle', 'off');
	colormap(gray); bar(1:C,hist(P.s,1:C),'stacked'); shading faceted; axis tight;
	title('FACTOR (CLASS) ASSIGNMENT (S) HISTOGRAM'); xlabel('s_{gn} value'); ylabel('count'); legend([repmat('n=',[N 1]) num2str([1:N]')]);
set(figure, 'name', 'DATA SET HISTOGRAM', 'NumberTitle', 'off');
	hist(x(:), 50); title('DATA SET (X) HISTOGRAM'); xlabel('x_{tg} value'); ylabel('count');
set(figure, 'name', 'FACTOR STRENGTH HISTOGRAM', 'NumberTitle', 'off');
	y = P.y(sub2ind([G C], repmat([1:G]',[1 N]),P.s)); for n = 2:N, y(find(P.r<n),n) = NaN; end;
	colormap(gray); bar(0:.2:5,hist(y,0:.2:5),'stacked'); shading faceted; axis tight; clear y;
	title('FACTOR STRENGTH (Y) HISTOGRAM'); xlabel('y_{gs_{gn}} value'); ylabel('count'); legend([repmat('n=',[N 1]) num2str([1:N]')]);
set(figure, 'name', 'FACTORS HISTOGRAM', 'NumberTitle', 'off');
	hist(P.z(:), 50); title('FACTORS (Z) HISTOGRAM'); xlabel('z_{tc} value'); ylabel('count');
set(figure, 'name', 'SENSOR NOISE HISTOGRAM', 'NumberTitle', 'off');
	hist(P.psi(:), 50); title('SENSOR NOISE (\Psi) HISTOGRAM'); xlabel('\psi_g value'); ylabel('count');